import React from 'react';
import { graphql, } from 'react-apollo';
import gql from 'graphql-tag';
import PropTypes from 'prop-types';

const ProjectLink = ({ project, }) => (
  <span>{project}</span>
);

ProjectLink.propTypes = {
  project: PropTypes.string.isRequired,
};

const SourceLink = ({ codeURL, title, }) => (
  <a href={codeURL} target="_blank" rel="noopener noreferrer">{title}</a>
);

SourceLink.propTypes = {
  codeURL: PropTypes.string.isRequired,
  title: PropTypes.string.isRequired,
};

const TextResultShape = PropTypes.shape({
  id: PropTypes.string,
  snippet: PropTypes.string,
  codeURL: PropTypes.string,
  project: PropTypes.string,
  fileName: PropTypes.string,
});

const ClassResultShape = PropTypes.shape({
  id: PropTypes.string,
  snippet: PropTypes.string,
  name: PropTypes.string,
  codeURL: PropTypes.string,
  project: PropTypes.string,
  fileName: PropTypes.string,
});

const UsageResultShape = PropTypes.shape({
  id: PropTypes.string,
  snippet: PropTypes.string,
  codeURL: PropTypes.string,
  project: PropTypes.string,
  fileName: PropTypes.string,
});

const DependencyRollupsGroupShape = PropTypes.shape({
  version: PropTypes.string,
  projects: PropTypes.arrayOf(PropTypes.string),
});

const DependencyRollupsShape = PropTypes.shape({
  name: PropTypes.string,
  direct: PropTypes.arrayOf(DependencyRollupsGroupShape),
  development: PropTypes.arrayOf(DependencyRollupsGroupShape),
  indirect: PropTypes.arrayOf(DependencyRollupsGroupShape),
});

const DependencyGroup = ({ group, title, }) => (
  <div>
    <h5>{title}</h5>
    <table width="100%">
      <tbody>
        {
          group.map(({ version, projects, }) => (
            <tr
              key={[
                version,
                title,
              ].join(':')}
            >
              <td width="10%">{version}</td>
              <td width="90%">
                {projects.map((project, index) => (
                  <span
                    key={[
                      project,
                      version,
                      title,
                    ].join(':')}
                  >
                    {index > 0 ? ' | ' : null}
                    <ProjectLink
                      project={project}
                    />
                  </span>
                ))}</td>
            </tr>
          ))
        }
      </tbody>
    </table>
  </div>
);

DependencyGroup.propTypes = {
  group: PropTypes.arrayOf(DependencyRollupsGroupShape).isRequired,
  title: PropTypes.string.isRequired,
};

const DependencyItem = ({ dependency, }) => (
  <div>
    <h4>{dependency.name}</h4>
    {dependency.direct.length > 0 ?
      <DependencyGroup
        group={dependency.direct}
        title="Direct"
      /> : null}
    {dependency.development.length > 0 ?
      <DependencyGroup
        group={dependency.development}
        title="Development"
      /> : null}
    {dependency.indirect.length > 0 ?
      <DependencyGroup
        group={dependency.indirect}
        title="Indirect"
      /> : null}
  </div>
);

DependencyItem.propTypes = {
  dependency: DependencyRollupsShape.isRequired,
};

const SearchResultsDependencies = ({ dependencies, }) => (
  <div>
    <h2>Dependencies</h2>
    {
      dependencies.map(dep => (
        <DependencyItem dependency={dep} key={dep.name} />
      ))
    }
  </div>
);

SearchResultsDependencies.propTypes = {
  dependencies: PropTypes.arrayOf(DependencyRollupsShape).isRequired,
};

const SearchResultsUsages = ({ usages, }) => (
  <div>
    <h2>Class Usage</h2>
    {usages.map(item => (
      <div key={item.id}>
        <ProjectLink project={item.project} />
        &nbsp; | &nbsp;
        <SourceLink codeURL={item.codeURL} title={item.fileName} />
        <pre>
          {item.snippet}
        </pre>
      </div>
    ))}
  </div>
);

SearchResultsUsages.propTypes = {
  usages: PropTypes.arrayOf(UsageResultShape),
};

SearchResultsUsages.defaultProps = {
  usages: [],
};

const SearchResultsClasses = ({ classes, }) => (
  <div>
    <h2>Classes</h2>
    {classes.map(item => (
      <div key={item.id}>
        <h4>{item.name}</h4>
        <ProjectLink project={item.project} />
        &nbsp; | &nbsp;
        <SourceLink codeURL={item.codeURL} title={item.fileName} />
        <pre>
          {item.snippet}
        </pre>
      </div>
    ))}
  </div>
);

SearchResultsClasses.propTypes = {
  classes: PropTypes.arrayOf(ClassResultShape),
};

SearchResultsClasses.defaultProps = {
  classes: [],
};

const SearchResultsText = ({ text, }) => (
  <div>
    <h2>Text</h2>
    {text.map(item => (
      <div key={item.id}>
        <ProjectLink project={item.project} />
        &nbsp; | &nbsp;
        <SourceLink codeURL={item.codeURL} title={item.fileName} />
        <pre>
          {item.snippet}
        </pre>
      </div>
    ))}
  </div>
);

SearchResultsText.propTypes = {
  text: PropTypes.arrayOf(TextResultShape),
};

SearchResultsText.defaultProps = {
  text: [],
};

const SearchResults = ({ text, classes, componentUsage, dependencies, }) => (
  <div>
    {classes.length > 0 ?
      <SearchResultsClasses
        classes={classes}
      /> : null}
    {text.length > 0 ?
      <SearchResultsText
        text={text}
      /> : null }
    {componentUsage.length > 0 ?
      <SearchResultsUsages
        usages={componentUsage}
      /> : null}
    {dependencies.length > 0 ?
      <SearchResultsDependencies
        dependencies={dependencies}
      /> : null}
  </div>
);

SearchResults.propTypes = {
  text: PropTypes.arrayOf(TextResultShape),
  classes: PropTypes.arrayOf(ClassResultShape),
  componentUsage: PropTypes.arrayOf(UsageResultShape),
  dependencies: PropTypes.arrayOf(DependencyRollupsShape),
};

SearchResults.defaultProps = {
  text: [],
  classes: [],
  componentUsage: [],
  dependencies: [],
};

const PROJECTS_GQL = gql`
query ($text: String) {
  search(text: $text) {
    text {
      project
      fileName
      id
      snippet
      codeURL
    }
    classes {
      project
      name
      id
      fileName
      snippet
      codeURL
    }
    componentUsage {
      start
      end
      project
      name
      id
      fileName
      snippet
      codeURL
    }
    dependencies {
      name
      direct {
        version
        projects
      }
      development {
        version
        projects
      }
      indirect {
        version
        projects
      }
    }
  }
}
`;

const WrappedSearchResults = graphql(PROJECTS_GQL, {
  props: ({ data: { search, }, }) => ({
    text: search ? search.text : [],
    classes: search ? search.classes : [],
    componentUsage: search ? search.componentUsage : [],
    dependencies: search ? search.dependencies : [],
  }),
  options: props => ({
    variables: {
      text: props.text,
    },
  }),
})(SearchResults);

WrappedSearchResults.SearchResults = SearchResults;

export default WrappedSearchResults;
